"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractIncluded = exports.extractData = void 0;
const camelize_1 = require("./camelize");
const extractResourceObjects = (data) => {
    if (Array.isArray(data)) {
        return data.map((datumn) => datumn.id);
    }
    return data.id;
};
function extractData(data, mode) {
    const modeType = mode || 'flatten';
    const { id, attributes, relationships } = data;
    const result = Object.assign({ id }, attributes);
    const extractedRelationships = extractRelationships(relationships, mode);
    if (modeType === 'flatten') {
        Object.assign(result, extractedRelationships);
    }
    else {
        Object.assign(result, {
            relationships: extractedRelationships,
        });
    }
    return camelize_1.camelizeKeys(result);
}
exports.extractData = extractData;
const extractRelationships = (data = {}, mode = 'flatten') => {
    return Object.entries(data).reduce((result, [k, v]) => {
        if (!(v === null || v === void 0 ? void 0 : v.data)) {
            return result;
        }
        const camelizedKey = camelize_1.camelize(k);
        if (mode === 'flatten') {
            result[camelizedKey] = extractResourceObjects(v.data);
        }
        else {
            result[camelizedKey] = v.data;
        }
        return result;
    }, {});
};
const extractIncluded = (data = [], mode = 'flatten') => {
    return data.reduce((result, resource) => {
        const { id, type } = resource;
        const datumn = extractData(resource, mode);
        const camelizedType = camelize_1.camelize(type);
        if (result[camelizedType]) {
            result[camelizedType][id] = datumn;
        }
        else {
            result[camelizedType] = {
                [id]: datumn,
            };
        }
        return result;
    }, {});
};
exports.extractIncluded = extractIncluded;
